<?php
/*
Plugin Name: WMPlayer
Version: 1.0.
Plugin URI: https://wmplayer.biz/faq/view/integration-with-wordpress
Author: wmplayer
Author URI: https://wmplayer.biz/
Description: WMPlayer for WordPress
Text Domain: wmplayer
 */
if (!defined('ABSPATH')) {
    exit;
}
if (!class_exists('WMPLAYER')) {

    /**
     * Class WMPLAYER
     */
    class WMPLAYER
    {
        const PAGE_SETTINGS = 'options-general.php?page=wmplayer-settings';
        const SETTING_NAME = 'wmplayer_options';

        const SETTING_CND_URL = 'wmplayer_cnd_url';
        const SETTING_RENDER_BLOCK_YOUTUBE = 'wmplayer_youtube';
        const SETTING_RENDER_BLOCK_CORE_VIDEO = 'wmplayer_core_video';

        public $plugin_version = '1.0.0';

        public function __construct()
        {
            define('WMPLAYER_VERSION', $this->plugin_version);
            $this->init();
        }

        /**
         * init plugin
         */
        private function init()
        {
            if (is_admin()) {
                add_filter('plugin_action_links', [$this, 'setActionLinks'], 10, 2);
                add_action('admin_menu', [$this, 'setAdminMenu']);
                add_action('admin_init', [$this, 'setAdminPluginSettings']);
            }

            add_action('wp_enqueue_scripts', [$this, 'setCommonScript']);

            add_action('plugins_loaded', [$this, 'setHandler']);

            add_shortcode('video', [$this, 'doShortCodeProcess']);
            add_shortcode('wmplayer', [$this, 'doShortCodeProcess']);

            add_filter('render_block', [$this, 'doRenderCodeProcess'], 11, 2);

            add_filter('widget_text', 'do_shortcode');
            add_filter('the_excerpt', 'do_shortcode', 11);
            add_filter('the_content', 'do_shortcode', 11);
        }

        /**
         * attach menu settings for plugins list
         * @param $links
         * @param $file
         * @return mixed
         */
        public function setActionLinks($links, $file)
        {
            if ($file == plugin_basename(dirname(__FILE__) . '/wmplayer.php')) {
                $links[] = sprintf(
                    '<a href="%s">%s</a>',
                    self::PAGE_SETTINGS,
                    __('Settings', 'WMPlayer')
                );
            }
            return $links;
        }

        public function setHandler()
        {
        }

        /**
         * render setting form
         */
        public function renderContentSetting()
        {
            ?>
            <form method="post" enctype="multipart/form-data" action="<?= self::PAGE_SETTINGS ?>">
                <?php
                settings_fields(self::SETTING_NAME);
                do_settings_sections(self::PAGE_SETTINGS);
                submit_button();
                ?>
            </form>
            <h2>Shortcodes and params</h2>
            <ul>
                <li>[video src="" poster=""] or [wmplayer src="" poster=""]</li>
            </ul>


            <?php
        }

        private function shortcodeAtts($atts = [])
        {
            return shortcode_atts([
                'src' => '',
                'url' => '',
                'file' => '',
                'width' => '',
                'controls' => '',
                'preload' => 'auto',
                'autoplay' => 'false',
                'loop' => '',
                'muted' => '',
                'poster' => '',
                'className' => '',
                'class' => '',
            ], $atts);
        }

        private function getGeneratePlayerId()
        {
            $uuid = bin2hex(random_bytes(7));
            return sprintf('player%s', $uuid);
        }

        private function getUrlWithAtts($atts = [])
        {
            if ($atts['src'] !== '') {
                return $atts['src'];
            }
            if ($atts['url'] !== '') {
                return $atts['url'];
            }
            if ($atts['file'] !== '') {
                return $atts['file'];
            }
            return '';
        }


        public function doRenderCodeProcess($blockContent, $block)
        {
            $blockName = $block['blockName'];

            if ('core-embed/youtube' === $blockName) {
                $replace = (int)get_option(self::SETTING_RENDER_BLOCK_YOUTUBE, 1) === 1;
                if ($replace) {
                    return $this->doShortCodeProcess($block['attrs']);
                }
            }

            if ('core-embed/videopress' === $blockName) {
                return $this->doShortCodeProcess($block['attrs']);
            }

            if ('core/video' === $blockName && class_exists('DOMDocument')) {
                $replace = (int)get_option(self::SETTING_RENDER_BLOCK_CORE_VIDEO, 1) === 1;
                if ($replace) {
                    $dom = new DOMDocument;
                    libxml_use_internal_errors(true);
                    $dom->loadHTML($block['innerHTML'], LIBXML_NOWARNING);
                    libxml_clear_errors();
                    $xpath = new DOMXPath($dom);
                    $videos = $dom->getElementsByTagName('video');
                    $result = [];
                    foreach ($videos as $video) {
                        $result[] = $this->doShortCodeProcess([
                            'src' => $video->getAttribute('src'),
                            'poster' => $video->getAttribute('poster'),
                        ]);
                    }
                    if (count($result) > 0) {
                        return implode("\n", $result);
                    }
                }
            }
            return $blockContent;
        }

        public function doShortCodeProcess($atts = [], $shortType = '')
        {
            $atts = $this->shortcodeAtts($atts);
            $playerId = $this->getGeneratePlayerId();
            $url = $this->getUrlWithAtts($atts);

            if ($url === '') {
                return 'you need to specify the src of the video file';
            }

            $tag = sprintf('<div id="%s"></div>', $playerId);

            $script = sprintf('<script defer type="application/javascript">
                (function(){
                  let player = new WMPlayer({
                    id: "%s",
                    file: "%s",
                    poster: "%s"
                  });
                  player.ready();
                })();
            </script>', $playerId, $url, $atts['poster']);

            return "{$tag}{$script}";
        }

        /**
         * attach page settings
         */
        public function setAdminPluginSettings()
        {
            add_settings_section('wmplayer_general_section', sprintf('WMPlayer Settings (v%s)', $this->plugin_version), '', self::PAGE_SETTINGS);

            register_setting(self::SETTING_NAME, self::SETTING_CND_URL);
            add_settings_field(
                self::SETTING_CND_URL,
                'CND Url',
                [$this, 'renderSettingsText'],
                self::PAGE_SETTINGS,
                'wmplayer_general_section', [
                    'type' => 'text',
                    'id' => self::SETTING_CND_URL,
                    'desc' => 'Set Player CND url or Local url',
                    'label_for' => self::SETTING_CND_URL
                ]
            );

            register_setting(self::SETTING_NAME, self::SETTING_RENDER_BLOCK_YOUTUBE);
            add_settings_field(
                self::SETTING_RENDER_BLOCK_YOUTUBE,
                'Replace render block/youtube',
                [$this, 'renderSettingsChecked'],
                self::PAGE_SETTINGS,
                'wmplayer_general_section', [
                    'type' => 'text',
                    'id' => self::SETTING_RENDER_BLOCK_YOUTUBE,
                    'desc' => '',
                    'label_for' => self::SETTING_RENDER_BLOCK_YOUTUBE
                ]
            );

            register_setting(self::SETTING_NAME, self::SETTING_RENDER_BLOCK_CORE_VIDEO);
            add_settings_field(
                self::SETTING_RENDER_BLOCK_CORE_VIDEO,
                'Replace render block/core video',
                [$this, 'renderSettingsChecked'],
                self::PAGE_SETTINGS,
                'wmplayer_general_section', [
                    'type' => 'text',
                    'id' => self::SETTING_RENDER_BLOCK_CORE_VIDEO,
                    'desc' => '',
                    'label_for' => self::SETTING_RENDER_BLOCK_CORE_VIDEO
                ]
            );

        }

        public function setCommonScript()
        {
            if (is_admin()) {
                return;
            }
            $url = get_option(self::SETTING_CND_URL);

            if ($url) {
                wp_register_script('wmplayer', $url);
                wp_enqueue_script('wmplayer');
            }

        }

        public function replaceNativeVideo()
        {
            printf('<script defer type="application/javascript"></script>');
        }

        /**
         * render input text
         * @param $args
         */
        public function renderSettingsText($args)
        {
            $id = $args['id'];
            $desc = isset($args['desc']) ? $args['desc'] : '';
            $value = get_option($id);
            if (array_key_exists('submit', $_POST) && array_key_exists($id, $_POST)) {
                $value = trim($_POST[$id]);
                update_option($id, $value);
            }
            printf('<input type="text" id="%s" name="%s" value="%s"  size="50" /><div style="margin-top: 10px;" class="desc">%s</div>',
                $id,
                $id,
                esc_attr($value),
                $desc
            );
        }

        /**
         * render input checkbox
         * @param $args
         */
        public function renderSettingsChecked($args)
        {
            $id = $args['id'];
            $desc = isset($args['desc']) ? $args['desc'] : '';
            $value = (int)get_option($id, 0);
            if (array_key_exists('submit', $_POST) && array_key_exists($id, $_POST)) {
                $value = (int)($_POST[$id]);
                update_option($id, $value);
            }
            printf('<input type="hidden" name="%s" />', $id);
            printf('<input type="checkbox" id="%s" name="%s" value="1" %s /><div style="margin-top: 10px;" class="desc">%s</div>',
                $id,
                $id,
                (int)$value === 1 ? ' checked' : '',
                $desc
            );

        }

        /**
         * attach wmplayer menu for settings
         */
        public function setAdminMenu()
        {
            if (is_admin()) {
                add_options_page('WMPlayer Setting',
                    'WMPlayer Setting',
                    'manage_options', 'wmplayer-settings', array($this, 'renderContentSetting')
                );
            }
        }

    }

    // init
    $GLOBALS['wmplayer'] = new WMPLAYER();
}